using System;
using System.Collections.Generic;
using System.Data;
using System.Data.SqlClient;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using SystemX.Product.ALIS.UI;
using static SystemX.Product.CP.TRA.MainForm;

namespace SystemX.Product.ALIS.Interface
{
    public enum eDbTableList
    {
        NONE,
        STAT_Host,
        STAT_User,
        STAT_TestCode,

        PROD_Group,
        PROD_TestList,
        PROD_Release,

        VRFY_Release,
        HIST_LatestStepVersion,

        HIST_LatestAccessKey,
        HIST_Summary,
        HIST_TestList,
        HIST_TestResult,
        HIST_TestResultShrotTerm,
        HIST_TestResultLongTerm
    }
    public interface IDataController
    {
        SqlCommand GetConnSqlCmd();

        bool GetDuplicateCheck(string strTableName, string strFieldName, string strValue);

        UserInfo GetLoginInfo();

        //TODO : DB CONTROL
        DataTable GetTable(string strQueryText);

        DataTable GetTable(eDbTableList tblName);

        DataTable GetTable(eDbTableList tblName, string strColName, string strParameter);

        string GetServerDateTimeString();

        DateTime GetServerDateTime();

        string SetConvertDateTime(string strGetConvertText);
    }
}
